@@ -1,17 +1,22 @@
 package e2e
 
 import (
+	"bufio"
 	"bytes"
 	goctx "context"
 	"encoding/json"
 	"fmt"
 	"io"
 	"os"
+	"os/exec"
 	"path"
+	"strconv"
 	"strings"
 	"testing"
 	"time"
 
+	v1 "k8s.io/api/rbac/v1"
+
 	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/runtime"
 
@@ -53,6 +58,12 @@ const (
 	defaultTestGracePeriod  = 20
 )
 
+const (
+	curlCMD    = "curl -ks -H \"Authorization: Bearer `cat /var/run/secrets/kubernetes.io/serviceaccount/token`\" "
+	metricsURL = "https://metrics.openshift-file-integrity.svc:8585/"
+	curlFIOCMD = curlCMD + metricsURL + "metrics-fio"
+)
+
 var mcLabelForWorkerRole = map[string]string{
 	mcWorkerRoleLabelKey: "worker",
 }
@@ -76,6 +87,7 @@ CONTENT_EX = sha512+ftype+p+u+g+n+acl+selinux+xattrs
 /hostroot/boot/        CONTENT_EX
 /hostroot/root/\..* PERMS
 /hostroot/root/   CONTENT_EX
+!/hostroot/root/\.kube
 !/hostroot/usr/src/
 !/hostroot/usr/tmp/
 
@@ -94,6 +106,9 @@ CONTENT_EX = sha512+ftype+p+u+g+n+acl+selinux+xattrs
 !/hostroot/etc/selinux/targeted
 !/hostroot/etc/openvswitch/conf.db
 !/hostroot/etc/kubernetes/cni/net.d/*
+!/hostroot/etc/machine-config-daemon/currentconfig$
+!/hostroot/etc/pki/ca-trust/extracted/java/cacerts$
+!/hostroot/etc/cvo/updatepayloads
 
 # Catch everything else in /etc
 /hostroot/etc/    CONTENT_EX`
@@ -185,13 +200,20 @@ func setupFileIntegrityOperatorCluster(t *testing.T, ctx *framework.Context) {
 	if err != nil {
 		t.Fatalf("failed to initialize cluster resources: %v", err)
 	}
-	t.Log("Initialized cluster resources")
+
+	// get global framework variables
+	f := framework.Global
 	namespace, err := ctx.GetOperatorNamespace()
 	if err != nil {
 		t.Fatal(err)
 	}
-	// get global framework variables
-	f := framework.Global
+
+	err = initializeMetricsTestResources(f, namespace)
+	if err != nil {
+		t.Fatalf("failed to initialize cluster resources for metrics: %v", err)
+	}
+
+	t.Log("Initialized cluster resources")
 	// wait for file-integrity-operator to be ready
 	err = e2eutil.WaitForOperatorDeployment(t, f.KubeClient, namespace, "file-integrity-operator", 1, retryInterval, timeout)
 	if err != nil {
@@ -199,6 +221,61 @@ func setupFileIntegrityOperatorCluster(t *testing.T, ctx *framework.Context) {
 	}
 }
 
+// Initializes the permission resources needed for the in-test metrics scraping
+func initializeMetricsTestResources(f *framework.Framework, namespace string) error {
+	if _, err := f.KubeClient.RbacV1().ClusterRoles().Create(goctx.TODO(), &v1.ClusterRole{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "fio-metrics-client",
+		},
+		Rules: []v1.PolicyRule{
+			{
+				NonResourceURLs: []string{
+					"/metrics-fio",
+				},
+				Verbs: []string{
+					"get",
+				},
+			},
+		},
+	}, metav1.CreateOptions{}); err != nil && !kerr.IsAlreadyExists(err) {
+		return err
+	}
+
+	if _, err := f.KubeClient.RbacV1().ClusterRoleBindings().Create(goctx.TODO(), &v1.ClusterRoleBinding{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "fio-metrics-client",
+		},
+		Subjects: []v1.Subject{
+			{
+				Kind:      "ServiceAccount",
+				Name:      "default",
+				Namespace: namespace,
+			},
+		},
+		RoleRef: v1.RoleRef{
+			APIGroup: "rbac.authorization.k8s.io",
+			Kind:     "ClusterRole",
+			Name:     "fio-metrics-client",
+		},
+	}, metav1.CreateOptions{}); err != nil && !kerr.IsAlreadyExists(err) {
+		return err
+	}
+
+	if _, err := f.KubeClient.CoreV1().Secrets(namespace).Create(goctx.TODO(), &corev1.Secret{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      "metrics-token",
+			Namespace: namespace,
+			Annotations: map[string]string{
+				"kubernetes.io/service-account.name": "default",
+			},
+		},
+		Type: "kubernetes.io/service-account-token",
+	}, metav1.CreateOptions{}); err != nil && !kerr.IsAlreadyExists(err) {
+		return err
+	}
+	return nil
+}
+
 // setupTest deploys the operator
 func setupTest(t *testing.T) (*framework.Framework, *framework.Context, string) {
 	testctx := setupTestRequirements(t)
@@ -228,7 +305,7 @@ func setupFileIntegrity(t *testing.T, f *framework.Framework, testCtx *framework
 
 	t.Logf("Created FileIntegrity: %+v", testIntegrityCheck)
 
-	dsName := common.GetDaemonSetName(testIntegrityCheck.Name)
+	dsName := common.DaemonSetName(testIntegrityCheck.Name)
 	err = waitForDaemonSet(daemonSetIsReady(f.KubeClient, dsName, namespace))
 	if err != nil {
 		t.Fatalf("Timed out waiting for DaemonSet %s", dsName)
@@ -336,9 +413,10 @@ func cleanAideAddedFilesDaemonset(namespace string) *appsv1.DaemonSet {
 
 // this daemonset is to stuff the aide log with added files, to force compression.
 // 20000 might not be enough
+// lowered to 10000
 func addLotsOfFilesDaemonset(namespace string) *appsv1.DaemonSet {
 	return privCommandDaemonset(namespace, "aide-add-lots-of-files",
-		"for i in `seq 1 20000`; do mktemp \"/hostroot/etc/addedbytest$i.XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\"; done || true",
+		"for i in `seq 1 10000`; do mktemp \"/hostroot/etc/addedbytest$i.XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\"; done || true",
 	)
 }
 
@@ -427,7 +505,8 @@ func getNumberOfWorkerNodes(c kubernetes.Interface) (int, error) {
 }
 
 // TODO: break this up into setupTest/setupFileIntegrity steps like below.
-func setupTolerationTest(t *testing.T, integrityName string) (*framework.Framework, *framework.Context, string) {
+func setupTolerationTest(t *testing.T, integrityName string) (*framework.Framework, *framework.Context, string, string) {
+	taintedNodeName := ""
 	testctx := setupTestRequirements(t)
 	namespace, err := testctx.GetOperatorNamespace()
 	if err != nil {
@@ -439,6 +518,7 @@ func setupTolerationTest(t *testing.T, integrityName string) (*framework.Framewo
 		t.Errorf("could not list nodes: %v", err)
 	}
 	taintedNode := &workerNodes[0]
+	taintedNodeName = taintedNode.Name
 	taintKey := "fi-e2e"
 	taintVal := "val"
 	taint := corev1.Taint{
@@ -490,7 +570,7 @@ func setupTolerationTest(t *testing.T, integrityName string) (*framework.Framewo
 		t.Errorf("could not create fileintegrity object: %v", err)
 	}
 
-	dsName := common.GetDaemonSetName(testIntegrityCheck.Name)
+	dsName := common.DaemonSetName(testIntegrityCheck.Name)
 	err = waitForDaemonSet(daemonSetIsReady(f.KubeClient, dsName, namespace))
 	if err != nil {
 		t.Errorf("Timed out waiting for DaemonSet %s", dsName)
@@ -514,7 +594,7 @@ func setupTolerationTest(t *testing.T, integrityName string) (*framework.Framewo
 		t.Errorf("error confirming DS replica amount: (%v) (%v)", pollErr, lastErr)
 	}
 
-	return f, testctx, namespace
+	return f, testctx, namespace, taintedNodeName
 }
 
 func updateFileIntegrityConfig(t *testing.T, f *framework.Framework, integrityName, configMapName, namespace, key string, interval, timeout time.Duration) {
@@ -639,7 +719,6 @@ func waitForFailedResultForNode(t *testing.T, f *framework.Framework, namespace,
 		nodeStatus := &fileintv1alpha1.FileIntegrityNodeStatus{}
 		getErr := f.Client.Get(goctx.TODO(), types.NamespacedName{Name: name + "-" + node, Namespace: namespace}, nodeStatus)
 		if getErr != nil {
-			t.Logf("Retrying. Got error: %v\n", getErr)
 			return false, nil
 		}
 
@@ -652,7 +731,7 @@ func waitForFailedResultForNode(t *testing.T, f *framework.Framework, namespace,
 		return false, nil
 	})
 	if err != nil {
-		t.Logf("status.nodeStatus for node %s not seen \n", node)
+		t.Logf("%s did not encounter failed condition", node)
 		return nil, err
 	}
 
@@ -694,14 +773,14 @@ func waitForFIStatusEvent(t *testing.T, f *framework.Framework, namespace, name,
 
 func assertNodeOKStatusEvents(t *testing.T, f *framework.Framework, namespace string, interval, timeout time.Duration) {
 	var lastErr error
-	nodes, err := f.KubeClient.CoreV1().Nodes().List(goctx.TODO(), metav1.ListOptions{LabelSelector: mcWorkerRoleLabelKey})
+	nodes, err := f.KubeClient.CoreV1().Nodes().List(goctx.TODO(), metav1.ListOptions{LabelSelector: nodeWorkerRoleLabelKey})
 	if err != nil {
 		t.Error(err)
 		return
 	}
 
 	seenNodes := map[string]bool{}
-	timeoutErr := wait.Poll(interval, timeout, func() (bool, error) {
+	timeoutErr := wait.PollImmediate(interval, timeout, func() (bool, error) {
 		// Node names are the key
 		eventList, getEventErr := f.KubeClient.CoreV1().Events(namespace).List(goctx.TODO(), metav1.ListOptions{
 			FieldSelector: "reason=NodeIntegrityStatus",
@@ -750,25 +829,22 @@ func assertNodesConditionIsSuccess(t *testing.T, f *framework.Framework, integri
 		Condition     fileintv1alpha1.FileIntegrityNodeCondition
 	}
 
-	nodes, err := f.KubeClient.CoreV1().Nodes().List(goctx.TODO(), metav1.ListOptions{LabelSelector: mcWorkerRoleLabelKey})
+	nodes, err := f.KubeClient.CoreV1().Nodes().List(goctx.TODO(), metav1.ListOptions{LabelSelector: nodeWorkerRoleLabelKey})
 	if err != nil {
 		t.Errorf("error listing nodes: %v", err)
 	}
-
-	timeoutErr := wait.Poll(interval, timeout, func() (bool, error) {
+	timeoutErr := wait.PollImmediate(interval, timeout, func() (bool, error) {
 		// Node names are the key
 		latestStatuses := map[string]nodeStatus{}
-
 		for _, node := range nodes.Items {
 			fileIntegNodeStatus := &fileintv1alpha1.FileIntegrityNodeStatus{}
 			getErr := f.Client.Get(goctx.TODO(), types.NamespacedName{Name: integrityName + "-" + node.Name, Namespace: namespace}, fileIntegNodeStatus)
 			if getErr != nil {
-				t.Logf("Retrying. Got error: %v\n", getErr)
 				lastErr = getErr
 				return false, nil
 			}
 
-			t.Logf("assertNodesConditionIsSuccess: found nodeStatus: %#v", fileIntegNodeStatus)
+			t.Logf("%s %s", fileIntegNodeStatus.NodeName, fileIntegNodeStatus.LastResult.Condition)
 
 			latestStatuses[fileIntegNodeStatus.NodeName] = nodeStatus{
 				LastProbeTime: fileIntegNodeStatus.LastResult.LastProbeTime,
@@ -796,6 +872,59 @@ func assertNodesConditionIsSuccess(t *testing.T, f *framework.Framework, integri
 	}
 }
 
+func assertSingleNodeConditionIsSuccess(t *testing.T, f *framework.Framework, integrityName, nodeName, namespace string, interval, timeout time.Duration) {
+	var lastErr error
+	type nodeStatus struct {
+		LastProbeTime metav1.Time
+		Condition     fileintv1alpha1.FileIntegrityNodeCondition
+	}
+
+	nodes, err := f.KubeClient.CoreV1().Nodes().List(goctx.TODO(), metav1.ListOptions{LabelSelector: nodeWorkerRoleLabelKey})
+	if err != nil {
+		t.Errorf("error listing nodes: %v", err)
+	}
+
+	timeoutErr := wait.PollImmediate(interval, timeout, func() (bool, error) {
+		// Node names are the key
+		latestStatuses := map[string]nodeStatus{}
+		for _, node := range nodes.Items {
+			if node.Name != nodeName {
+				continue
+			}
+			fileIntegNodeStatus := &fileintv1alpha1.FileIntegrityNodeStatus{}
+			getErr := f.Client.Get(goctx.TODO(), types.NamespacedName{Name: integrityName + "-" + node.Name, Namespace: namespace}, fileIntegNodeStatus)
+			if getErr != nil {
+				lastErr = getErr
+				return false, nil
+			}
+
+			t.Logf("%s %s", fileIntegNodeStatus.NodeName, fileIntegNodeStatus.LastResult.Condition)
+
+			latestStatuses[fileIntegNodeStatus.NodeName] = nodeStatus{
+				LastProbeTime: fileIntegNodeStatus.LastResult.LastProbeTime,
+				Condition:     fileIntegNodeStatus.LastResult.Condition,
+			}
+		}
+
+		// iterate gathered statuses
+		for name, status := range latestStatuses {
+			if status.Condition != fileintv1alpha1.NodeConditionSucceeded {
+				lastErr = fmt.Errorf("status.nodeStatus for node %s NOT SUCCESS: But instead %s", name, status.Condition)
+				return false, nil
+			}
+		}
+		// reset error since we're good
+		lastErr = nil
+		return true, nil
+	})
+	if lastErr != nil {
+		t.Errorf("ERROR: nodes weren't in good state: %s", lastErr)
+	}
+	if timeoutErr != nil {
+		t.Errorf("ERROR: timed out waiting for node condition: %s", timeoutErr)
+	}
+}
+
 // waitForScanStatus will poll until the fileintegrity that we're looking for reaches a certain status, or until
 // a timeout is reached.
 func waitForScanStatus(t *testing.T, f *framework.Framework, namespace, name string, targetStatus fileintv1alpha1.FileIntegrityStatusPhase) error {
@@ -814,7 +943,6 @@ func pollUntilConfigMapDataMatches(t *testing.T, f *framework.Framework, namespa
 	return wait.PollImmediate(interval, timeout, func() (bool, error) {
 		cm, getErr := f.KubeClient.CoreV1().ConfigMaps(namespace).Get(goctx.TODO(), name, metav1.GetOptions{})
 		if getErr != nil {
-			t.Logf("Retrying. Got error: %v\n", getErr)
 			return false, nil
 		}
 		if cm.Data[key] == expected {
@@ -829,7 +957,6 @@ func pollUntilConfigMapExists(t *testing.T, f *framework.Framework, namespace, n
 	err := wait.PollImmediate(interval, timeout, func() (bool, error) {
 		cm, getErr := f.KubeClient.CoreV1().ConfigMaps(namespace).Get(goctx.TODO(), name, metav1.GetOptions{})
 		if getErr != nil {
-			t.Logf("Retrying. Got error: %v\n", getErr)
 			return false, nil
 		}
 		data = cm.Data
@@ -973,7 +1100,7 @@ func getTestMcfg(t *testing.T) *mcfgv1.MachineConfig {
 	return mcfg
 }
 
-func waitForNodesToBeReady(f *framework.Framework) error {
+func waitForNodesToBeReady(t *testing.T, f *framework.Framework) error {
 	nodeList := &corev1.NodeList{}
 	// A long time...
 	bo := backoff.WithMaxRetries(backoff.NewConstantBackOff(15*time.Second), 360)
@@ -995,8 +1122,7 @@ func waitForNodesToBeReady(f *framework.Framework) error {
 		},
 		bo,
 		func(err error, d time.Duration) {
-			// TODO(jaosorior): Change this for a log call
-			fmt.Printf("Nodes not ready yet after %s: %s\n", d.String(), err)
+			t.Logf("Nodes not ready yet after %s: %s\n", d.String(), err)
 		})
 	if err != nil {
 		return fmt.Errorf("The nodes were never ready: %s", err)
@@ -1017,7 +1143,7 @@ func isNodeReady(node corev1.Node) bool {
 
 func assertDSPodHasArg(t *testing.T, f *framework.Framework, fiName, namespace, expectedLine string, interval, timeout time.Duration) error {
 	return wait.PollImmediate(interval, timeout, func() (bool, error) {
-		ds, getErr := f.KubeClient.AppsV1().DaemonSets(namespace).Get(goctx.TODO(), common.GetDaemonSetName(fiName), metav1.GetOptions{})
+		ds, getErr := f.KubeClient.AppsV1().DaemonSets(namespace).Get(goctx.TODO(), common.DaemonSetName(fiName), metav1.GetOptions{})
 		if getErr != nil {
 			t.Logf("Retrying. Got error: %v\n", getErr)
 			return false, nil
@@ -1033,7 +1159,7 @@ func assertDSPodHasArg(t *testing.T, f *framework.Framework, fiName, namespace,
 }
 
 func getFiDsPods(f *framework.Framework, fileIntegrityName, namespace string) (*corev1.PodList, error) {
-	dsName := common.GetDaemonSetName(fileIntegrityName)
+	dsName := common.DaemonSetName(fileIntegrityName)
 	ds, err := f.KubeClient.AppsV1().DaemonSets(namespace).Get(goctx.TODO(), dsName, metav1.GetOptions{})
 	if err != nil {
 		return nil, err
@@ -1118,7 +1244,7 @@ func getNodesWithSelector(f *framework.Framework, labelselector map[string]strin
 	return nodes.Items, err
 }
 
-func writeToArtifactsDir(t *testing.T, f *framework.Framework, dir, scan, pod, container, log string) error {
+func writeToArtifactsDir(dir, scan, pod, container, log string) error {
 	logPath := path.Join(dir, fmt.Sprintf("%s_%s_%s.log", scan, pod, container))
 	logFile, err := os.Create(logPath)
 	if err != nil {
@@ -1170,7 +1296,7 @@ func logContainerOutput(t *testing.T, f *framework.Framework, namespace, name st
 				if len(logs) == 0 {
 					t.Logf("no logs for %s/%s", pod.Name, con)
 				} else {
-					err := writeToArtifactsDir(t, f, artifacts, name, pod.Name, con, logs)
+					err := writeToArtifactsDir(artifacts, name, pod.Name, con, logs)
 					if err != nil {
 						t.Logf("error writing logs for %s/%s: %v", pod.Name, con, err)
 					} else {
@@ -1181,3 +1307,68 @@ func logContainerOutput(t *testing.T, f *framework.Framework, namespace, name st
 		}
 	}
 }
+
+func getMetricResults(t *testing.T) string {
+	ocPath, err := exec.LookPath("oc")
+	if err != nil {
+		t.Fatal(err)
+	}
+	// We're just under test.
+	// G204 (CWE-78): Subprocess launched with variable (Confidence: HIGH, Severity: MEDIUM)
+	// #nosec
+	cmd := exec.Command(ocPath,
+		"run", "--rm", "-i", "--restart=Never", "--image=registry.fedoraproject.org/fedora-minimal:latest",
+		"-nopenshift-file-integrity", "metrics-test", "--", "bash", "-c",
+		curlFIOCMD,
+	)
+	out, err := cmd.CombinedOutput()
+	if err != nil {
+		t.Errorf("error getting output %s", err)
+	}
+	t.Logf("metrics output:\n%s\n", string(out))
+	return string(out)
+}
+
+func assertMetric(t *testing.T, content, metric string, expected int) error {
+	if val := parseMetric(t, content, metric); val != expected {
+		return errors.New(fmt.Sprintf("expected %v for counter %s, got %v", expected, metric, val))
+	}
+	return nil
+}
+
+func assertEachMetric(t *testing.T, expectedMetrics map[string]int) error {
+	metricErrs := make([]error, 0)
+	metricsOutput := getMetricResults(t)
+	for metric, i := range expectedMetrics {
+		err := assertMetric(t, metricsOutput, metric, i)
+		if err != nil {
+			metricErrs = append(metricErrs, err)
+		}
+	}
+	if len(metricErrs) > 0 {
+		for err := range metricErrs {
+			t.Log(err)
+		}
+		return errors.New("unexpected metrics value")
+	}
+	return nil
+}
+
+func parseMetric(t *testing.T, content, metric string) int {
+	scanner := bufio.NewScanner(strings.NewReader(content))
+	for scanner.Scan() {
+		line := scanner.Text()
+		if strings.HasPrefix(line, metric) {
+			fields := strings.Fields(line)
+			if len(fields) != 2 {
+				t.Errorf("invalid metric")
+			}
+			i, err := strconv.Atoi(fields[1])
+			if err != nil {
+				t.Errorf("invalid metric value")
+			}
+			return i
+		}
+	}
+	return 0
+}
